@@ -13,21 +13,21 @@ from sklearn.svm import SVC
 
 class NmfModel:
     def __init__(
-        self,
-        mat: Union[np.ndarray, pd.DataFrame],
-        ncomp: int,
-        nblocks: int = 1,
-        name: str = "",
-        use_hals: bool = True,
-        beta_loss: str = "frobenius",
-        leverage: str = "standard",
-        kernel: str = "linear",
-        tol: float = 1e-6,
-        max_iter: int = 150,
-        max_iter_mult: int = 20,
-        regularization: str = None,
-        features_names: Tuple[str] = None,
-        object_names: Tuple[str] = None,
+            self,
+            mat: Union[np.ndarray, pd.DataFrame],
+            ncomp: int,
+            nblocks: int = 1,
+            name: str = "",
+            use_hals: bool = True,
+            beta_loss: str = "frobenius",
+            leverage: str = "standard",
+            kernel: str = "linear",
+            tol: float = 1e-6,
+            max_iter: int = 150,
+            max_iter_mult: int = 20,
+            regularization: str = None,
+            features_names: Tuple[str] = None,
+            object_names: Tuple[str] = None,
     ):
 
         if object_names is None:
@@ -85,6 +85,8 @@ class NmfModel:
             self._nobj, self._nfeat = mat.shape
 
         self._filledmat = self._mat.copy()
+        # Check
+        self._comp = max(1, min(self._ncomp, min(self._filledmat.shape)))
 
         if name == "":
             self._name = "NMF_" + str(self._ncomp) + "(O" + str(self._nobj) + ",F" + str(self._nfeat) + ")"
@@ -197,9 +199,9 @@ class NmfModel:
 
     def is_ok(self):
         return False if not hasattr(self, "_estimator") or not hasattr(self, "_predictor") \
-            or self._predictor is None \
-            or "H" not in list(self._predictor.keys()) \
-            or self._predictor["H"].shape[1] != self._ncomp \
+                        or self._predictor is None \
+                        or "H" not in list(self._predictor.keys()) \
+                        or self._predictor["H"].shape[1] != self._ncomp \
             else True
 
     @property
@@ -274,8 +276,6 @@ class NmfModel:
         df_hhi_1 = self.get_h_hhi_1(do_sort=False)
         return np.mean(df_hhi_1)
 
-
-
     def get_features_clustering_proba(self):
         if not self.is_ok():
             self.test_model()
@@ -292,6 +292,14 @@ class NmfModel:
         else:
             return np.zeros(self._ncomp)
 
+    def get_clustering_stability(self, which="W") -> float:
+        if not self.is_ok():
+            self.test_model()
+        nbins = min(self._ncomp, 5)
+        df_clust = pd.DataFrame(self._estimator["WB"] if which.lower() == "w" else self._estimator["HB"])
+        entropies = [utl.normalized_entropy(df_clust.loc[idx, :], nbins) for idx in df_clust.index]
+        return float(np.nanmean(entropies))
+
     def get_null_features(self):
         ht = self.get_h().T
         _, to_drop_features = utl.drop_zero_columns(ht)
@@ -304,7 +312,7 @@ class NmfModel:
         if rank < 0:
             delta_hhi_1 = cols_hhi_1.diff(1).iloc[1:]
             rank = np.argmin(delta_hhi_1)
-        comp_list = cols_hhi_1.index[:rank+1]
+        comp_list = cols_hhi_1.index[:rank + 1]
         fuzzy_features = []
         if len(comp_list) > 0:
             ht = self.get_h().T
@@ -324,7 +332,7 @@ class NmfModel:
         else:
             return 0.0
 
-    def get_precision(self) -> float:
+    def get_precision(self, relative=True) -> float:
         # if not self.is_ok():
         #         #     self.test_model()
         #         # if 'diff' in self._estimator:
@@ -335,7 +343,8 @@ class NmfModel:
             self.test_model()
         m_proxy = self.get_m_proxy()
         m_delta = self._mat - m_proxy
-        return utl.mean_square(m_delta) / utl.mean_square(self._mat)
+        denom = utl.mean_square(self._mat) if relative else 1.0
+        return utl.mean_square(m_delta) / denom
 
     def get_volume(self) -> float:
         if not self.is_ok():
@@ -345,6 +354,18 @@ class NmfModel:
         else:
             return np.nan
 
+    def get_aic(self, absolute: bool=True) -> float:
+        if not self.is_ok():
+            self.test_model()
+        freeparams = self._ncomp * (self._nobj + self._nfeat)
+        msize = self._nobj * self._nfeat
+        # errsq is the mean squared error between the given matrix and its nmf proxy
+        errsq = self.get_precision(relative=False)
+        res = 2 * (freeparams + msize * (errsq + 0.5 * math.log(errsq)) - 1.0)
+        if absolute:
+            res /= msize
+        return res
+
     def get_h_compression(self):
         if not self.is_ok():
             self.test_model()
@@ -400,9 +421,10 @@ class NmfModel:
     def get_sorted_features(self):
         return list(self._sortedmat.columns)
 
-    def get_obs_leverage_clusters(self):
+    def get_obs_leverage_clusters(self) -> pd.DataFrame:
         """
         Attribue à chaque ligne de W un cluster dans [1, self._ncomp], en prenant en compte le 'leverage'
+        Renvoie la liste des numéros de clusters
         """
         if not self.is_ok():
             self.test_model()
@@ -417,7 +439,7 @@ class NmfModel:
         wmat = self.get_w()
         ngr = ngroups if ngroups > 0 else self._ncomp
         km = KMeans(ngr)
-        return pd.DataFrame(index=self._objnames, columns=["Observation Cluster"], data=km.fit_predict(wmat)+1)
+        return pd.DataFrame(index=self._objnames, columns=["Observation Cluster"], data=km.fit_predict(wmat) + 1)
 
     def get_observations_clusters_incidence_matrix(self, method='leverage', df_clusters_priors=None):
         """
@@ -577,7 +599,7 @@ class NmfModel:
         volumes = []
         ratios = []
 
-        for n in range(n_lower, n_upper+1):
+        for n in range(n_lower, n_upper + 1):
             self.set_ncomp(n)
             self.test_model()
             precision = self.get_precision()
@@ -589,10 +611,9 @@ class NmfModel:
 
         df = pd.DataFrame({'precision': precisions,
                            'volume': volumes,
-                           'ratio': ratios}, index=range(n_lower, n_upper+1))
+                           'ratio': ratios}, index=range(n_lower, n_upper + 1))
         return df
 
-
     @property
     def errors(self):
         return self._errors
@@ -646,50 +667,102 @@ def compare_nmf_fs_to_random(train_set: Union[np.ndarray, pd.DataFrame],
 
 
 def generate_scree_plot(mat: Union[np.ndarray, pd.DataFrame],
-                        ncmin: int, ncmax: int):
+                        ncmin: int, ncmax: int, known_ncomp=0,
+                        do_entropies: bool = False,
+                        do_stabilities: bool = True) -> pd.DataFrame:
+    """
+    Generates a scree plot, ie a collection of metrics for the NMF factorization of a given matrix realized for
+    various number of components.
+
+    Parameters
+    ----------
+    mat: Union[np.ndarray, pd.DataFrame]
+        The matrix to factorize
+    ncmin: int
+        the minimum tested number of components
+    ncmax: int
+        the maximum tested number of components
+    known_ncomp: int = 0
+        the 'true' number of components
+    do_entropies: bool = False
+        if True, calculate the average diagonal entropies. INCREASE COMPUTATION TIME
+    do_stabilities: bool = True
+        if True, calculate the stability of W and H clustering. Requires several boostraps (10 by default) performed
+        by the underlying NmfModel object.
+
+    Returns
+    -------
+    A dataframe indexed by the tested components, with quality metrics in columns
+    """
     columns = []
     ncomp = ncmin
     nmfmodel = NmfModel(mat=mat, ncomp=ncomp, regularization="components")
+    nmfmodel.set_n_bootstrap(10)
     n, f = mat.shape
     df_scree_plot = pd.DataFrame(columns=columns)
-    for ncomp in range(ncmin, ncmax+1):
+    for ncomp in range(ncmin, ncmax + 1):
         nmfmodel.set_ncomp(ncomp)
+        nmfmodel.set_n_bootstrap(10)
         line = pd.DataFrame(index=[ncomp], columns=columns)
         err = line.loc[ncomp, "Error"] = nmfmodel.get_precision()
         vol = line.loc[ncomp, "Volume"] = nmfmodel.get_volume()
+        line.loc[ncomp, "Known_ncomp"] = known_ncomp
         line.loc[ncomp, "Tried_ncomp"] = ncomp
-        line.loc[ncomp, "Obs_feat_ratio"] = float(n) / float(f)
+        line.loc[ncomp, "AIC"] = nmfmodel.get_aic()
+        line.loc[ncomp, "w_stability"] = nmfmodel.get_clustering_stability(which="W") if do_stabilities else np.nan
+        line.loc[ncomp, "h_stability"] = nmfmodel.get_clustering_stability(which="H") if do_stabilities else np.nan
         line.loc[ncomp, "Err_Vol_Ratio"] = err / vol if vol > 0 else 1e6
         line.loc[ncomp, "W_clust_hhi_1"] = nmfmodel.get_observations_clusters_hhi_1()
         line.loc[ncomp, "H_clust_hhi_1"] = nmfmodel.get_features_clusters_hhi_1()
-        line.loc[ncomp, "Avg_relative_entropy"] = nmfmodel.get_avg_diag_entropies(nbins=5,
-                                                                                  size_weighted=True,
-                                                                                  relative=True).iloc[0, 0]
+        entrop = nmfmodel.get_avg_diag_entropies(nbins=5, size_weighted=True, relative=True).iloc[0, 0] \
+            if do_entropies else 0.0
+        line.loc[ncomp, "Avg_relative_entropy"] = entrop
         df_scree_plot = pd.concat([df_scree_plot, line], axis=0)
     return df_scree_plot
 
 
-def add_miniscree_plots_from_scree_plot(df_examples: pd.DataFrame,
-                                        scree_plot: pd.DataFrame,
-                                        known_ncomp: int,
-                                        window_size: int = 6,
-                                        dropna: bool = True):
+def collect_windows_from_scree_plot(df_collected_windows: pd.DataFrame,
+                                    df_scree_plot: pd.DataFrame,
+                                    known_ncomp: int = 0,
+                                    window_size: int = 6,
+                                    dropna: bool = True,
+                                    ncomp_margin=-1):
+    """
+    Collects windows of given size (default 6) from a unique scree plot and copies the result in a collective dataframe.
+    Parameters
+    ----------
+    df_collected_windows: pd.DataFrame
+        The collection of results for all windows. Results are stacked vertically, i.e along the index.
+    df_scree_plot: pd.DataFrame
+        The NMF scree plot of a matrix for a range of components
+    known_ncomp: int = 0
+        If positive, the known number of components associated to the matrix
+    window_size: int = 6
+        The size of the window. Metrics will be collected in  [c, c + window_size - 1]
+    dropna: bool = True:
+        If true, all lines containing an empty cell are dropped
+    ncomp_margin: int = -1
+        If positive or 0, metrics are only collected in [k - m, k + w + m -1] where:
+            k = known_comp, m = margin, w = windows_size
+        If negative, metrics are collected for all components
+    Returns
+    -------
+        The collection dataframe stacked with new metrics windows
+
+    """
+    nb_examples, _ = df_collected_windows.shape
+    duplicated_examples = utl.duplicate_data_wfold(df=df_scree_plot, window=window_size)
 
-    nb_examples, _ = df_examples.shape
-    duplicated_examples = utl.duplicate_data_wfold(df=scree_plot, window=window_size)
     if dropna:
         duplicated_examples.dropna(axis=0, how='any', inplace=True)
-    for ncomp in scree_plot.index:
-        position = 0 if known_ncomp <= ncomp else window_size-1 if known_ncomp >= ncomp + window_size - 1 \
-                else known_ncomp - ncomp
-
-        duplicated_examples.loc[ncomp, "Position"] = position
+    for ncomp in df_scree_plot.index:
+        position = 0 if known_ncomp <= ncomp else window_size - 1 if known_ncomp >= ncomp + window_size - 1 \
+            else known_ncomp - ncomp
+        if ncomp_margin < 0 or \
+                ncomp - ncomp_margin <= known_ncomp <= ncomp + window_size - 1 + ncomp_margin:
+            duplicated_examples.loc[ncomp, "Position"] = position
     duplicated_examples.index += nb_examples
-    df_examples = pd.concat([df_examples, duplicated_examples], axis=0)
+    df_collected_windows = pd.concat([df_collected_windows, duplicated_examples], axis=0)
     if dropna:
-        df_examples.dropna(axis=0, how='any', inplace=True)
-    return df_examples
-
-
-
-
+        df_collected_windows.dropna(axis=0, how='any', inplace=True)
+    return df_collected_windows
