@@ -772,7 +772,7 @@ def generate_clusters_centroids(ndim: int, max_corr: float, n_centroids: int = 0
 
     Returns
     -------
-
+    A ndim x ndim dataframe. The desired vectors are in columns and satsify the correlation condition.
     """
     assert (ndim >= 1)
     assert (0 <= max_corr < 1)
@@ -834,7 +834,8 @@ def generate_clusterized_vectors(vsize: int, nbvect: int, nclusters: int,
     nbvect: int
         total number of vector to generate
     nclusters: int
-        number of vector clusters to generate
+        number of vector clusters to generate. The default case is nclusters == vsize,
+        ie as many clusters as the dimension
     clusterminsize: int
         minimum size of a cluster
     max_inter_corr: float
